@@ -8,7 +8,7 @@ require 'ostruct'
 # * t - the table types to test. ie: myisam, innodb, memory, temporary, etc.
 #
 module BenchmarkOptionParser
-  BANNER = "Usage: ruby #{$0} [options]\nSee ruby #{$0} -h for more options."
+  BANNER = "Usage: ruby #{$0} [options]\nSee ruby #{$0} -h for more options.".freeze
 
   def self.print_banner
     puts BANNER
@@ -24,13 +24,13 @@ module BenchmarkOptionParser
     puts "  Database adapter: #{options.adapter}"
     puts "  Number of objects: #{options.number_of_objects}"
     puts "  Table types:"
-    print_valid_table_types( options, :prefix=>"    " )
+    print_valid_table_types( options, prefix: "    " )
   end
 
   # TODO IMPLEMENT THIS
-  def self.print_valid_table_types( options, hsh={:prefix=>''} )
-    if options.table_types.keys.size > 0
-      options.table_types.keys.sort.each{ |type| puts hsh[:prefix].to_s + type.to_s }
+  def self.print_valid_table_types( options, hsh = { prefix: '' } )
+    if !options.table_types.keys.empty?
+      options.table_types.keys.sort.each { |type| puts hsh[:prefix].to_s + type.to_s }
     else
       puts 'No table types defined.'
     end
@@ -38,13 +38,13 @@ module BenchmarkOptionParser
 
   def self.parse( args )
     options = OpenStruct.new(
-      :adapter => 'mysql',
-      :table_types => {},
-      :delete_on_finish => true,
-      :number_of_objects => [],
-      :outputs => [] )
+      adapter: 'mysql2',
+      table_types: {},
+      delete_on_finish: true,
+      number_of_objects: [],
+      outputs: [] )
 
-    opts = OptionParser.new do |opts|
+    opt_parser = OptionParser.new do |opts|
       opts.banner = BANNER
 
       # parse the database adapter
@@ -55,8 +55,8 @@ module BenchmarkOptionParser
 
       # parse do_not_delete flag
       opts.on( "d", "--do-not-delete",
-        "By default all records in the benchmark tables will be deleted at the end of the benchmark. " +
-        "This flag indicates not to delete the benchmark data." ) do |arg|
+        "By default all records in the benchmark tables will be deleted at the end of the benchmark. " \
+        "This flag indicates not to delete the benchmark data." ) do |_|
         options.delete_on_finish = false
       end
 
@@ -78,30 +78,29 @@ module BenchmarkOptionParser
 
       # print results in CSV format
       opts.on( "--to-csv [String]", "Print results in a CSV file format" ) do |filename|
-        options.outputs << OpenStruct.new( :format=>'csv', :filename=>filename)
+        options.outputs << OpenStruct.new( format: 'csv', filename: filename)
       end
 
       # print results in HTML format
       opts.on( "--to-html [String]", "Print results in HTML format" ) do |filename|
-        options.outputs << OpenStruct.new( :format=>'html', :filename=>filename )
+        options.outputs << OpenStruct.new( format: 'html', filename: filename )
       end
-    end #end opt.parse!
+    end # end opt.parse!
 
     begin
-      opts.parse!( args )
-      if options.table_types.size == 0
+      opt_parser.parse!( args )
+      if options.table_types.empty?
         options.table_types['all'] = options.benchmark_all_types = true
       end
-    rescue Exception => ex
+    rescue Exception
       print_banner!
     end
 
     options.number_of_objects = [1000] if options.number_of_objects.empty?
-    options.outputs = [ OpenStruct.new( :format => 'html', :filename => 'benchmark.html')] if options.outputs.empty?
+    options.outputs = [OpenStruct.new( format: 'html', filename: 'benchmark.html')] if options.outputs.empty?
 
     print_options( options )
 
     options
   end
-
 end
