@@ -1,3 +1,3 @@
 class TestInnoDb < ActiveRecord::Base
-  set_table_name 'test_innodb'
+  self.table_name = 'test_innodb'
 end
