@@ -3,7 +3,7 @@ module ActiveRecord::Import::MysqlAdapter
   include ActiveRecord::Import::OnDuplicateKeyUpdateSupport
 
   NO_MAX_PACKET = 0
-  QUERY_OVERHEAD = 8 #This was shown to be true for MySQL, but it's not clear where the overhead is from.
+  QUERY_OVERHEAD = 8 # This was shown to be true for MySQL, but it's not clear where the overhead is from.
 
   # +sql+ can be a single string or an array. If it is an array all
   # elements that are in position >= 1 will be appended to the final SQL.
@@ -11,19 +11,19 @@ module ActiveRecord::Import::MysqlAdapter
     # the number of inserts default
     number_of_inserts = 0
 
-    base_sql,post_sql = if sql.is_a?( String )
-      [ sql, '' ]
+    base_sql, post_sql = if sql.is_a?( String )
+      [sql, '']
     elsif sql.is_a?( Array )
-      [ sql.shift, sql.join( ' ' ) ]
+      [sql.shift, sql.join( ' ' )]
     end
 
     sql_size = QUERY_OVERHEAD + base_sql.size + post_sql.size
 
     # the number of bytes the requested insert statement values will take up
-    values_in_bytes = values.sum {|value| value.bytesize }
+    values_in_bytes = values.sum(&:bytesize)
 
     # the number of bytes (commas) it will take to comma separate our values
-    comma_separated_bytes = values.size-1
+    comma_separated_bytes = values.size - 1
 
     # the total number of bytes required if this statement is one statement
     total_bytes = sql_size + values_in_bytes + comma_separated_bytes
@@ -31,22 +31,22 @@ module ActiveRecord::Import::MysqlAdapter
     max = max_allowed_packet
 
     # if we can insert it all as one statement
-    if NO_MAX_PACKET == max or total_bytes < max
+    if NO_MAX_PACKET == max || total_bytes < max
       number_of_inserts += 1
       sql2insert = base_sql + values.join( ',' ) + post_sql
       insert( sql2insert, *args )
     else
       value_sets = ::ActiveRecord::Import::ValueSetsBytesParser.parse(values,
-        :reserved_bytes => sql_size,
-        :max_bytes => max)
-      value_sets.each do |values|
+        reserved_bytes: sql_size,
+        max_bytes: max)
+      value_sets.each do |value_set|
         number_of_inserts += 1
-        sql2insert = base_sql + values.join( ',' ) + post_sql
+        sql2insert = base_sql + value_set.join( ',' ) + post_sql
         insert( sql2insert, *args )
       end
     end
 
-    [number_of_inserts,[]]
+    [number_of_inserts, []]
   end
 
   # Returns the maximum number of bytes that the server will allow
@@ -60,6 +60,19 @@ module ActiveRecord::Import::MysqlAdapter
     end
   end
 
+  # Add a column to be updated on duplicate key update
+  def add_column_for_on_duplicate_key_update( column, options = {} ) # :nodoc:
+    if options.include?(:on_duplicate_key_update)
+      columns = options[:on_duplicate_key_update]
+      case columns
+      when Array then columns << column.to_sym unless columns.include?(column.to_sym)
+      when Hash then columns[column.to_sym] = column.to_sym
+      end
+    else
+      options[:on_duplicate_key_update] = [column.to_sym]
+    end
+  end
+
   # Returns a generated ON DUPLICATE KEY UPDATE statement given the passed
   # in +args+.
   def sql_for_on_duplicate_key_update( table_name, *args ) # :nodoc:
@@ -72,12 +85,12 @@ module ActiveRecord::Import::MysqlAdapter
     elsif arg.is_a?( String )
       sql << arg
     else
-      raise ArgumentError.new( "Expected Array or Hash" )
+      raise ArgumentError, "Expected Array or Hash"
     end
     sql
   end
 
-  def sql_for_on_duplicate_key_update_as_array( table_name, arr )  # :nodoc:
+  def sql_for_on_duplicate_key_update_as_array( table_name, arr ) # :nodoc:
     results = arr.map do |column|
       qc = quote_column_name( column )
       "#{table_name}.#{qc}=VALUES(#{qc})"
@@ -86,7 +99,6 @@ module ActiveRecord::Import::MysqlAdapter
   end
 
   def sql_for_on_duplicate_key_update_as_hash( table_name, hsh ) # :nodoc:
-    sql = ' ON DUPLICATE KEY UPDATE '
     results = hsh.map do |column1, column2|
       qc1 = quote_column_name( column1 )
       qc2 = quote_column_name( column2 )
@@ -95,8 +107,8 @@ module ActiveRecord::Import::MysqlAdapter
     results.join( ',')
   end
 
-  #return true if the statement is a duplicate key record error
-  def duplicate_key_update_error?(exception)# :nodoc:
+  # Return true if the statement is a duplicate key record error
+  def duplicate_key_update_error?(exception) # :nodoc:
     exception.is_a?(ActiveRecord::StatementInvalid) && exception.to_s.include?('Duplicate entry')
   end
 end
