@@ -3,13 +3,14 @@ require "active_record"
 require "active_record/version"
 
 module ActiveRecord::Import
-  AdapterPath = "activerecord-import/active_record/adapters"
+  ADAPTER_PATH = "activerecord-import/active_record/adapters".freeze
 
   def self.base_adapter(adapter)
     case adapter
-    when 'mysqlspatial' then 'mysql'
+    when 'mysql2_makara' then 'mysql2'
     when 'mysql2spatial' then 'mysql2'
     when 'spatialite' then 'sqlite3'
+    when 'postgresql_makara' then 'postgresql'
     when 'postgis' then 'postgresql'
     else adapter
     end
@@ -17,9 +18,9 @@ module ActiveRecord::Import
 
   # Loads the import functionality for a specific database adapter
   def self.require_adapter(adapter)
-    require File.join(AdapterPath,"/abstract_adapter")
+    require File.join(ADAPTER_PATH, "/abstract_adapter")
     begin
-      require File.join(AdapterPath,"/#{base_adapter(adapter)}_adapter")
+      require File.join(ADAPTER_PATH, "/#{base_adapter(adapter)}_adapter")
     rescue LoadError
       # fallback
     end
