@@ -23,13 +23,14 @@ describe "#import" do
     it "should not modify the passed in columns array" do
       assert_nothing_raised do
         columns = %w(title author_name).freeze
-        Topic.import columns, [["foo", "bar"]]
+        Topic.import columns, [%w(foo bar)]
       end
     end
 
     it "should not modify the passed in values array" do
       assert_nothing_raised do
-        values = [["foo", "bar"]].freeze
+        record = %w(foo bar).freeze
+        values = [record].freeze
         Topic.import %w(title author_name), values
       end
     end
@@ -47,26 +48,26 @@ describe "#import" do
 
   context "with :validation option" do
     let(:columns) { %w(title author_name) }
-    let(:valid_values) { [[ "LDAP", "Jerry Carter"], ["Rails Recipes", "Chad Fowler"]] }
-    let(:valid_values_with_context) { [[ 1111, "Jerry Carter"], [2222, "Chad Fowler"]] }
-    let(:invalid_values) { [[ "The RSpec Book", ""], ["Agile+UX", ""]] }
+    let(:valid_values) { [["LDAP", "Jerry Carter"], ["Rails Recipes", "Chad Fowler"]] }
+    let(:valid_values_with_context) { [[1111, "Jerry Carter"], [2222, "Chad Fowler"]] }
+    let(:invalid_values) { [["The RSpec Book", ""], ["Agile+UX", ""]] }
 
     context "with validation checks turned off" do
       it "should import valid data" do
         assert_difference "Topic.count", +2 do
-          result = Topic.import columns, valid_values, :validate => false
+          Topic.import columns, valid_values, validate: false
         end
       end
 
       it "should import invalid data" do
         assert_difference "Topic.count", +2 do
-          result = Topic.import columns, invalid_values, :validate => false
+          Topic.import columns, invalid_values, validate: false
         end
       end
 
       it 'should raise a specific error if a column does not exist' do
         assert_raises ActiveRecord::Import::MissingColumnError do
-          Topic.import ['foo'], [['bar']], :validate => false
+          Topic.import ['foo'], [['bar']], validate: false
         end
       end
     end
@@ -74,37 +75,37 @@ describe "#import" do
     context "with validation checks turned on" do
       it "should import valid data" do
         assert_difference "Topic.count", +2 do
-          result = Topic.import columns, valid_values, :validate => true
+          Topic.import columns, valid_values, validate: true
         end
       end
 
       it "should import valid data with on option" do
         assert_difference "Topic.count", +2 do
-          result = Topic.import columns, valid_values_with_context, :validate_with_context => :context_test
+          Topic.import columns, valid_values_with_context, validate_with_context: :context_test
         end
       end
 
       it "should not import invalid data" do
         assert_no_difference "Topic.count" do
-          result = Topic.import columns, invalid_values, :validate => true
+          Topic.import columns, invalid_values, validate: true
         end
       end
 
       it "should import invalid data with on option" do
         assert_no_difference "Topic.count" do
-          result = Topic.import columns, valid_values, :validate_with_context => :context_test
+          Topic.import columns, valid_values, validate_with_context: :context_test
         end
       end
 
       it "should report the failed instances" do
-        results = Topic.import columns, invalid_values, :validate => true
+        results = Topic.import columns, invalid_values, validate: true
         assert_equal invalid_values.size, results.failed_instances.size
-        results.failed_instances.each{ |e| assert_kind_of Topic, e }
+        results.failed_instances.each { |e| assert_kind_of Topic, e }
       end
 
       it "should import valid data when mixed with invalid data" do
         assert_difference "Topic.count", +2 do
-          result = Topic.import columns, valid_values + invalid_values, :validate => true
+          Topic.import columns, valid_values + invalid_values, validate: true
         end
         assert_equal 0, Topic.where(title: invalid_values.map(&:first)).count
       end
@@ -113,48 +114,64 @@ describe "#import" do
 
   context "with :all_or_none option" do
     let(:columns) { %w(title author_name) }
-    let(:valid_values) { [[ "LDAP", "Jerry Carter"], ["Rails Recipes", "Chad Fowler"]] }
-    let(:invalid_values) { [[ "The RSpec Book", ""], ["Agile+UX", ""]] }
+    let(:valid_values) { [["LDAP", "Jerry Carter"], ["Rails Recipes", "Chad Fowler"]] }
+    let(:invalid_values) { [["The RSpec Book", ""], ["Agile+UX", ""]] }
     let(:mixed_values) { valid_values + invalid_values }
 
     context "with validation checks turned on" do
       it "should import valid data" do
         assert_difference "Topic.count", +2 do
-          result = Topic.import columns, valid_values, :all_or_none => true
+          Topic.import columns, valid_values, all_or_none: true
         end
       end
 
       it "should not import invalid data" do
         assert_no_difference "Topic.count" do
-          result = Topic.import columns, invalid_values, :all_or_none => true
+          Topic.import columns, invalid_values, all_or_none: true
         end
       end
 
       it "should not import valid data when mixed with invalid data" do
         assert_no_difference "Topic.count" do
-          result = Topic.import columns, mixed_values, :all_or_none => true
+          Topic.import columns, mixed_values, all_or_none: true
         end
       end
 
       it "should report the failed instances" do
-        results = Topic.import columns, mixed_values, :all_or_none => true
+        results = Topic.import columns, mixed_values, all_or_none: true
         assert_equal invalid_values.size, results.failed_instances.size
         results.failed_instances.each { |e| assert_kind_of Topic, e }
       end
 
       it "should report the zero inserts" do
-        results = Topic.import columns, mixed_values, :all_or_none => true
+        results = Topic.import columns, mixed_values, all_or_none: true
         assert_equal 0, results.num_inserts
       end
     end
   end
 
+  context "with :batch_size option" do
+    it "should import with a single insert" do
+      assert_difference "Topic.count", +10 do
+        result = Topic.import Build(10, :topics), batch_size: 10
+        assert_equal 1, result.num_inserts if Topic.supports_import?
+      end
+    end
+
+    it "should import with multiple inserts" do
+      assert_difference "Topic.count", +10 do
+        result = Topic.import Build(10, :topics), batch_size: 4
+        assert_equal 3, result.num_inserts if Topic.supports_import?
+      end
+    end
+  end
+
   context "with :synchronize option" do
     context "synchronizing on new records" do
       let(:new_topics) { Build(3, :topics) }
 
       it "doesn't reload any data (doesn't work)" do
-        Topic.import new_topics, :synchronize => new_topics
+        Topic.import new_topics, synchronize: new_topics
         if Topic.support_setting_primary_key_of_imported_objects?
           assert new_topics.all?(&:persisted?), "Records should have been reloaded"
         else
@@ -167,7 +184,7 @@ describe "#import" do
       let(:new_topics) { Build(3, :topics) }
 
       it "reloads data for existing in-memory instances" do
-        Topic.import(new_topics, :synchronize => new_topics, :synchronize_keys => [:title] )
+        Topic.import(new_topics, synchronize: new_topics, synchronize_keys: [:title] )
         assert new_topics.all?(&:persisted?), "Records should have been reloaded"
       end
     end
@@ -176,21 +193,21 @@ describe "#import" do
       let(:new_topics) { Generate(3, :topics) }
 
       it "reloads data for existing in-memory instances" do
-        new_topics.each &:destroy
-        Topic.import(new_topics, :synchronize => new_topics, :synchronize_keys => [:title] )
+        new_topics.each(&:destroy)
+        Topic.import(new_topics, synchronize: new_topics, synchronize_keys: [:title] )
         assert new_topics.all?(&:persisted?), "Records should have been reloaded"
       end
     end
   end
 
   context "with an array of unsaved model instances" do
-    let(:topic) { Build(:topic, :title => "The RSpec Book", :author_name => "David Chelimsky")}
+    let(:topic) { Build(:topic, title: "The RSpec Book", author_name: "David Chelimsky") }
     let(:topics) { Build(9, :topics) }
-    let(:invalid_topics){ Build(7, :invalid_topics)}
+    let(:invalid_topics) { Build(7, :invalid_topics) }
 
     it "should import records based on those model's attributes" do
       assert_difference "Topic.count", +9 do
-        result = Topic.import topics
+        Topic.import topics
       end
 
       Topic.import [topic]
@@ -198,7 +215,7 @@ describe "#import" do
     end
 
     it "should not overwrite existing records" do
-      topic = Generate(:topic, :title => "foobar")
+      topic = Generate(:topic, title: "foobar")
       assert_no_difference "Topic.count" do
         begin
           Topic.transaction do
@@ -216,13 +233,13 @@ describe "#import" do
     context "with validation checks turned on" do
       it "should import valid models" do
         assert_difference "Topic.count", +9 do
-          result = Topic.import topics, :validate => true
+          Topic.import topics, validate: true
         end
       end
 
       it "should not import invalid models" do
         assert_no_difference "Topic.count" do
-          result = Topic.import invalid_topics, :validate => true
+          Topic.import invalid_topics, validate: true
         end
       end
     end
@@ -230,7 +247,7 @@ describe "#import" do
     context "with validation checks turned off" do
       it "should import invalid models" do
         assert_difference "Topic.count", +7 do
-          result = Topic.import invalid_topics, :validate => false
+          Topic.import invalid_topics, validate: false
         end
       end
     end
@@ -241,7 +258,7 @@ describe "#import" do
 
     it "should import records populating the supplied columns with the corresponding model instance attributes" do
       assert_difference "Topic.count", +2 do
-        result = Topic.import [:author_name, :title], topics
+        Topic.import [:author_name, :title], topics
       end
 
       # imported topics should be findable by their imported attributes
@@ -252,7 +269,7 @@ describe "#import" do
     it "should not populate fields for columns not imported" do
       topics.first.author_email_address = "zach.dennis@gmail.com"
       assert_difference "Topic.count", +2 do
-        result = Topic.import [:author_name, :title], topics
+        Topic.import [:author_name, :title], topics
       end
 
       assert !Topic.where(author_email_address: "zach.dennis@gmail.com").first
@@ -267,27 +284,29 @@ describe "#import" do
   end
 
   context "ActiveRecord timestamps" do
+    let(:time) { Chronic.parse("5 minutes ago") }
+
     context "when the timestamps columns are present" do
       setup do
         @existing_book = Book.create(title: "Fell", author_name: "Curry", publisher: "Bayer", created_at: 2.years.ago.utc, created_on: 2.years.ago.utc)
         ActiveRecord::Base.default_timezone = :utc
-        Timecop.freeze Chronic.parse("5 minutes ago") do
+        Timecop.freeze(time) do
           assert_difference "Book.count", +2 do
-            result = Book.import ["title", "author_name", "publisher", "created_at", "created_on"], [["LDAP", "Big Bird", "Del Rey", nil, nil], [@existing_book.title, @existing_book.author_name, @existing_book.publisher, @existing_book.created_at, @existing_book.created_on]]
+            Book.import %w(title author_name publisher created_at created_on), [["LDAP", "Big Bird", "Del Rey", nil, nil], [@existing_book.title, @existing_book.author_name, @existing_book.publisher, @existing_book.created_at, @existing_book.created_on]]
           end
         end
         @new_book, @existing_book = Book.last 2
       end
 
-      it "should set the created_at column for new records"  do
-        assert_equal 5.minutes.ago.utc.strftime("%H:%M"), @new_book.created_at.strftime("%H:%M")
+      it "should set the created_at column for new records" do
+        assert_in_delta time.to_i, @new_book.created_at.to_i, 1.second
       end
 
       it "should set the created_on column for new records" do
-        assert_equal 5.minutes.ago.utc.strftime("%H:%M"), @new_book.created_on.strftime("%H:%M")
+        assert_in_delta time.to_i, @new_book.created_on.to_i, 1.second
       end
 
-      it "should not set the created_at column for existing records"  do
+      it "should not set the created_at column for existing records" do
         assert_equal 2.years.ago.utc.strftime("%Y:%d"), @existing_book.created_at.strftime("%Y:%d")
       end
 
@@ -296,27 +315,25 @@ describe "#import" do
       end
 
       it "should set the updated_at column for new records" do
-        assert_equal 5.minutes.ago.utc.strftime("%H:%M"), @new_book.updated_at.strftime("%H:%M")
+        assert_in_delta time.to_i, @new_book.updated_at.to_i, 1.second
       end
 
       it "should set the updated_on column for new records" do
-        assert_equal 5.minutes.ago.utc.strftime("%H:%M"), @new_book.updated_on.strftime("%H:%M")
+        assert_in_delta time.to_i, @new_book.updated_on.to_i, 1.second
       end
     end
 
     context "when a custom time zone is set" do
-      let(:time){ Chronic.parse("5 minutes ago")  }
-
       setup do
         Timecop.freeze(time) do
           assert_difference "Book.count", +1 do
-            result = Book.import [:title, :author_name, :publisher], [["LDAP", "Big Bird", "Del Rey"]]
+            Book.import [:title, :author_name, :publisher], [["LDAP", "Big Bird", "Del Rey"]]
           end
         end
         @book = Book.last
       end
 
-      it "should set the created_at and created_on timestamps for new records"  do
+      it "should set the created_at and created_on timestamps for new records" do
         assert_in_delta time.to_i, @book.created_at.to_i, 1.second
         assert_in_delta time.to_i, @book.created_on.to_i, 1.second
       end
@@ -329,11 +346,11 @@ describe "#import" do
   end
 
   context "importing with database reserved words" do
-    let(:group) { Build(:group, :order => "superx") }
+    let(:group) { Build(:group, order: "superx") }
 
     it "should import just fine" do
       assert_difference "Group.count", +1 do
-        result = Group.import [group]
+        Group.import [group]
       end
       assert_equal "superx", Group.first.order
     end
@@ -347,50 +364,39 @@ describe "#import" do
   end
 
   context "importing through an association scope" do
-    [ true, false ].each do |b|
-      context "when validation is " + (b ? "enabled" : "disabled") do
-        it "should automatically set the foreign key column" do
-          books = [[ "David Chelimsky", "The RSpec Book" ], [ "Chad Fowler", "Rails Recipes" ]]
-          topic = FactoryGirl.create :topic
-          topic.books.import [ :author_name, :title ], books, :validate => b
-          assert_equal 2, topic.books.count
-          assert topic.books.all? { |b| b.topic_id == topic.id }
+    { has_many: :chapters, polymorphic: :discounts }.each do |association_type, association|
+      let(:book)   { FactoryGirl.create :book }
+      let(:scope)  { book.public_send association }
+      let(:klass)  { { chapters: Chapter, discounts: Discount }[association] }
+      let(:column) { { chapters: :title,  discounts: :amount  }[association] }
+      let(:val1)   { { chapters: 'A',     discounts: 5        }[association] }
+      let(:val2)   { { chapters: 'B',     discounts: 6        }[association] }
+
+      context "for #{association_type}" do
+        it "works importing models" do
+          scope.import [
+            klass.new(column => val1),
+            klass.new(column => val2)
+          ]
+
+          assert_equal [val1, val2], scope.map(&column).sort
+        end
+
+        it "works importing array of columns and values" do
+          scope.import [column], [[val1], [val2]]
+
+          assert_equal [val1, val2], scope.map(&column).sort
         end
       end
     end
-
-    it "works importing models" do
-      topic = FactoryGirl.create :topic
-      books = [
-        Book.new(:author_name => "Author #1", :title => "Book #1"),
-        Book.new(:author_name => "Author #2", :title => "Book #2"),
-      ]
-      topic.books.import books
-      assert_equal 2, topic.books.count
-      assert topic.books.detect { |b| b.title == "Book #1" && b.author_name == "Author #1" }
-      assert topic.books.detect { |b| b.title == "Book #2" && b.author_name == "Author #2" }
-    end
-
-    it "works importing array of columns and values" do
-      topic = FactoryGirl.create :topic
-      books = [
-        Book.new(:author_name => "Foo", :title => "Baz"),
-        Book.new(:author_name => "Foo2", :title => "Baz2"),
-      ]
-      topic.books.import [:author_name, :title], [["Author #1", "Book #1"], ["Author #2", "Book #2"]]
-      assert_equal 2, topic.books.count
-      assert topic.books.detect { |b| b.title == "Book #1" && b.author_name == "Author #1" }
-      assert topic.books.detect { |b| b.title == "Book #2" && b.author_name == "Author #2" }
-    end
-
   end
 
   context 'When importing models with Enum fields' do
     it 'should be able to import enum fields' do
       Book.delete_all if Book.count > 0
       books = [
-        Book.new(:author_name => "Foo", :title => "Baz", status: 0),
-        Book.new(:author_name => "Foo2", :title => "Baz2", status: 1),
+        Book.new(author_name: "Foo", title: "Baz", status: 0),
+        Book.new(author_name: "Foo2", title: "Baz2", status: 1),
       ]
       Book.import books
       assert_equal 2, Book.count
@@ -404,12 +410,27 @@ describe "#import" do
       end
     end
 
-    if ENV['AR_VERSION'].to_i > 4.1
+    it 'should be able to import enum fields with default value' do
+      Book.delete_all if Book.count > 0
+      books = [
+        Book.new(author_name: "Foo", title: "Baz")
+      ]
+      Book.import books
+      assert_equal 1, Book.count
+
+      if ENV['AR_VERSION'].to_i >= 5.0
+        assert_equal 'draft', Book.first.read_attribute('status')
+      else
+        assert_equal 0, Book.first.read_attribute('status')
+      end
+    end
+
+    if ENV['AR_VERSION'].to_f > 4.1
       it 'should be able to import enum fields by name' do
         Book.delete_all if Book.count > 0
         books = [
-          Book.new(:author_name => "Foo", :title => "Baz", status: :draft),
-          Book.new(:author_name => "Foo2", :title => "Baz2", status: :published),
+          Book.new(author_name: "Foo", title: "Baz", status: :draft),
+          Book.new(author_name: "Foo2", title: "Baz2", status: :published),
         ]
         Book.import books
         assert_equal 2, Book.count
@@ -425,6 +446,25 @@ describe "#import" do
     end
   end
 
+  context 'When importing arrays of values with Enum fields' do
+    let(:columns) { [:author_name, :title, :status] }
+    let(:values) { [['Author #1', 'Book #1', 0], ['Author #2', 'Book #2', 1]] }
+
+    it 'should be able to import enum fields' do
+      Book.delete_all if Book.count > 0
+      Book.import columns, values
+      assert_equal 2, Book.count
+
+      if ENV['AR_VERSION'].to_i >= 5.0
+        assert_equal 'draft', Book.first.read_attribute('status')
+        assert_equal 'published', Book.last.read_attribute('status')
+      else
+        assert_equal 0, Book.first.read_attribute('status')
+        assert_equal 1, Book.last.read_attribute('status')
+      end
+    end
+  end
+
   describe "importing when model has default_scope" do
     it "doesn't import the default scope values" do
       assert_difference "Widget.unscoped.count", +2 do
@@ -447,17 +487,42 @@ describe "#import" do
   describe "importing serialized fields" do
     it "imports values for serialized fields" do
       assert_difference "Widget.unscoped.count", +1 do
-        Widget.import [:w_id, :data], [[1, {:a => :b}]]
+        Widget.import [:w_id, :data], [[1, { a: :b }]]
       end
-      assert_equal({:a => :b}, Widget.find_by_w_id(1).data)
+      assert_equal({ a: :b }, Widget.find_by_w_id(1).data)
     end
 
-    requires_active_record_version ">= 4" do
+    if ENV['AR_VERSION'].to_f >= 3.1
+      let(:data) { { a: :b } }
       it "imports values for serialized JSON fields" do
         assert_difference "Widget.unscoped.count", +1 do
-          Widget.import [:w_id, :json_data], [[9, {:a => :b}]]
+          Widget.import [:w_id, :json_data], [[9, data]]
+        end
+        assert_equal(data.as_json, Widget.find_by_w_id(9).json_data)
+      end
+    end
+  end
+
+  describe "#import!" do
+    let(:columns) { %w(title author_name) }
+    let(:valid_values) { [["LDAP", "Jerry Carter"], ["Rails Recipes", "Chad Fowler"]] }
+    let(:invalid_values) { [["Rails Recipes", "Chad Fowler"], ["The RSpec Book", ""], ["Agile+UX", ""]] }
+
+    context "with invalid data" do
+      it "should raise ActiveRecord::RecordInvalid" do
+        assert_no_difference "Topic.count" do
+          assert_raise ActiveRecord::RecordInvalid do
+            Topic.import! columns, invalid_values
+          end
+        end
+      end
+    end
+
+    context "with valid data" do
+      it "should import data" do
+        assert_difference "Topic.count", +2 do
+          Topic.import! columns, valid_values
         end
-        assert_equal({:a => :b}.as_json, Widget.find_by_w_id(9).json_data)
       end
     end
   end
