@@ -1,7 +1,7 @@
 class Widget < ActiveRecord::Base
   self.primary_key = :w_id
 
-  default_scope lambda { where(active: true) }
+  default_scope -> { where(active: true) }
 
   serialize :data, Hash
   serialize :json_data, JSON
