@@ -1,11 +1,11 @@
 class ActiveSupport::TestCase
-  def Build(*args)
+  def Build(*args) # rubocop:disable Style/MethodName
     n = args.shift if args.first.is_a?(Numeric)
     factory = args.shift
     factory_girl_args = args.shift || {}
 
     if n
-      Array.new.tap do |collection|
+      [].tap do |collection|
         n.times.each { collection << FactoryGirl.build(factory.to_s.singularize.to_sym, factory_girl_args) }
       end
     else
@@ -13,13 +13,13 @@ class ActiveSupport::TestCase
     end
   end
 
-  def Generate(*args)
+  def Generate(*args) # rubocop:disable Style/MethodName
     n = args.shift if args.first.is_a?(Numeric)
     factory = args.shift
     factory_girl_args = args.shift || {}
 
     if n
-      Array.new.tap do |collection|
+      [].tap do |collection|
         n.times.each { collection << FactoryGirl.create(factory.to_s.singularize.to_sym, factory_girl_args) }
       end
     else
